//   -----   jeu.cpp   -----   //

#include "jeu.moc"

// Constructeur
KJeu :: KJeu ( ) : KMainWindow (  ), numCoup ( 1 )
{
   // Les coups jous
   listCoups = new QListView ( this, "Coups Jous : ", 0 );
   listCoups -> addColumn ( " N ");
   listCoups -> addColumn ( "Joueur : " );
   listCoups -> addColumn ( "Position : " );

   // Bouton Quitter la partie
   btnQuitter = new QPushButton ( "Quitter", this ) ;
   btnQuitter -> show ( ) ;

   connect ( btnQuitter, SIGNAL ( clicked ( ) ), this, SLOT ( slotbtnQuitter ( ) ) ) ;

   // Taille maximale
   tailleDamier = 14;

   lblScoreNoir = new QLabel ( this ) ;
   lblScoreNoir -> show ( ) ;

   lblScoreBlanc = new QLabel ( this ) ;
   lblScoreBlanc -> show ( ) ;

   lblTour = new QLabel ( this ) ;
   QPalette p ;
   p.setColor ( QPalette::Active, QColorGroup::Foreground, QColor ( 255, 0, 0 ) );
   p.setColor ( QPalette::Active, QColorGroup::Background, QColor ( 220, 220, 220 ) );
   p.setColor ( QPalette::Inactive, QColorGroup::Foreground, QColor ( 255, 0, 0 ) );
   p.setColor ( QPalette::Inactive, QColorGroup::Background, QColor ( 220, 220, 220 ) );
   lblTour -> setPalette ( p ) ;
   lblTour -> show ( ) ;

   // Allocation mmoire pour les pions du damier
   pions = new monPion *[ tailleDamier + 2 ] ;
   for ( int i = 0; i < tailleDamier + 2; i ++ )
      pions [ i ] = new monPion [ tailleDamier + 2 ] ;

   // Initialisation pour les dplacements autoriss
   pionsValides = new bool *[ tailleDamier + 2 ] ;
   for ( int i = 0; i < tailleDamier + 2; i ++ )
      pionsValides [ i ] = new bool [ tailleDamier + 2 ] ;

   getPosLigne ( 14 ) ;
   login="";
   adv="";
   id_partie="";
   id_temp="";
   demande=false;
   partieFinie=false;
   demandeEnCours=false;

   pthread_mutex_init(&mut,NULL);
}


// Destructeur
KJeu :: ~KJeu ( )
{
   for ( int i = 0; i < tailleDamier + 2; i++ )
      delete [ ] pions [ i ];
   delete [ ] pions ;
   for ( int i = 0; i < tailleDamier + 2; i++ )
      delete [ ] pionsValides [ i ];
   delete [ ] pionsValides ;
   pthread_mutex_destroy(&mut);
}


// ------------------------------------------------------------------------------------------------ //
//			Gestion des vnements
// ------------------------------------------------------------------------------------------------ //

// Redessine la fenetre
void KJeu :: paintEvent ( QPaintEvent * )
{
   QRect cr ( 25, 25, 25 * ( tailleDamier +1  ), 25 * ( tailleDamier + 1  ) ) ;
   QPixmap pix ( cr.size ());
   pix.fill ( backgroundColor () ) ;

   QPainter paint ;
   paint.begin ( &pix );

   QColor colNoir ( 0, 0, 0 ) ;
   QFont maFont ( "arial", 12, QFont :: Normal ) ;


   // Dessine les traits du damier
   for ( int j = 1; j <= tailleDamier; j++ )
      for ( int i = 1; i <= tailleDamier; i++ )
         paint.drawRect ( 25 * i, 25 * j, 25, 25 ) ;

   // Dessine les numros et les lettres du damier
   paint.setPen ( colNoir ) ;
   paint.setFont ( maFont ) ;
   QString tmp = "" ;

   for ( int i = 0; i < tailleDamier; i++ )
   {
      paint.drawText ( 7, 27 + ( 25 * i ), 25, 25 * tailleDamier, Qt :: AlignLeft, getPosLigne ( i +1 ) ) ;
      if ( i == 8 )    // Car la lettre 'I' prend moins de place
         tmp += " " ;
      paint.drawText ( 32 + ( 25 * i ), 7, 25 * tailleDamier, 25, Qt :: AlignLeft, tmp + getPosColonne ( i +1 ) ) ;
   }

   // Dessine les pions sans redessiner tous les pions
   for ( int j = 1; j <= tailleDamier; j++ )
      for ( int i = 1; i <= tailleDamier; i++ )
      {
            switch ( pions [ i ] [ j ].getCouleur ( ) )   // Suivant la couleur du pion on va le dessiner
            {
	        case RIEN :     if ( pionsValides [ i ] [ j ] )    // Si on a le  droit de jouer  cette position, on affiche une croix
                                        paint.drawImage ( 25 * i, 25 * j, pions [ i ] [ j ].dessinePion ( CROIX ), -1, -1, pions [ i ] [ j ].dessinePion ( CROIX ).width ( ), pions [ i ] [ j ].dessinePion ( CROIX ).height ( ) ) ;
                                     else      // sinon on affiche une case vide
                                       paint.drawImage ( 25 * i, 25 * j, pions [ i ] [ j ].dessinePion ( RIEN ), -1, -1, pions [ i ] [ j ].dessinePion ( RIEN ).width ( ), pions [ i ] [ j ].dessinePion ( RIEN ).height ( ) ) ;
	                             break ;
		case BLANC : paint.drawImage ( 25 * i, 25 * j, pions [ i ] [ j ].dessinePion ( BLANC ), -1, -1, pions [ i ] [ j ].dessinePion ( BLANC ).width ( ), pions [ i ] [ j ].dessinePion ( BLANC ).height ( ) ) ;
	                              break ;
	        case NOIR :    paint.drawImage ( 25 * i, 25 * j, pions [ i ] [ j ].dessinePion ( NOIR ), -1, -1, pions [ i ] [ j ].dessinePion ( NOIR ).width ( ), pions [ i ] [ j ].dessinePion ( NOIR ).height ( ) ) ;
	                             break ;
            }
      }

    // Affiche les 4 points de reper sur le damier
   QColor colPoints ( 0, 0, 0);
   paint.setBrush ( colPoints );
   paint.drawEllipse ( 72, 72, 6, 6 ) ;
   paint.drawEllipse ( 72, ( ( tailleDamier - 1 ) * 25 ) - 3, 6, 6 ) ;
   paint.drawEllipse ( ( ( tailleDamier - 1 ) * 25 ) - 3, 72, 6, 6 ) ;
   paint.drawEllipse ( ( ( tailleDamier - 1 ) * 25 ) - 3, ( ( tailleDamier - 1 ) * 25 ) - 3, 6, 6 ) ;

   if ( leCoup == maCouleur )
      lblTour -> setText ( i18n("My turn") );
   else
      lblTour -> setText ( i18n("Waiting ...") );

   bitBlt ( this, 0, 0, &pix );
   paint.end ( );
}


// Fermeture de l'application
void KJeu :: closeEvent ( QCloseEvent * )
{
  slotbtnQuitter ( ) ;
}


// Apercu de l'application
void KJeu :: showEvent ( QShowEvent * )
{
   numCoup = 1 ;
   listCoups -> clear ( ) ;
   listCoups -> setGeometry ( ( getTaille ( ) + 2 ) * 25, 25, 173, getTaille ( ) * 25 ) ;

   if (maCouleur == 0)
   {
      lblScoreNoir -> setText ( login + " (Noir) : 2" ) ;
      lblScoreBlanc -> setText ( adv + " (Blanc) : 2" ) ;
   }
   else
   {
      lblScoreNoir -> setText ( adv + " (Noir) : 2" ) ;
      lblScoreBlanc -> setText ( login + " (Blanc) : 2" ) ;
   }

   lblScoreNoir -> setMinimumWidth( lblScoreNoir->sizeHint().width() );
   lblScoreBlanc ->setMinimumWidth( lblScoreBlanc->sizeHint().width() );
   lblTour -> setMinimumWidth ( lblTour ->sizeHint().width() ) ;
   btnQuitter -> setGeometry ( ( getTaille ( ) * 25 ) + 170, ( getTaille ( ) * 25 ) + 50, 50, 25 ) ;
}


// Gestion du clic de souris sur la fenetre
void KJeu :: mousePressEvent ( QMouseEvent *ev )
{
   if ( ! coupLocal ( ) )   return ;

   // Gestion du clic dans le damier
   if ( ev -> x ( ) >= 25 && ev -> x ( ) <= ( ( tailleDamier + 1 ) * 25 ) )
   {
      if ( ev -> y ( ) >= 25 && ev -> y ( ) <= ( ( tailleDamier +1 ) * 25 ) )
      {
         pos_x = getPosition ( ev -> x ( ) ) ;
	 pos_y = getPosition ( ev -> y ( ) ) ;
	 valideClick ( leCoup, pos_x, pos_y ) ;
	 coups = "" ;
      }
  }
}


// ------------------------------------------------------------------------------------------------ //
//			Mthodes la classe KJeu
// ------------------------------------------------------------------------------------------------ //

// Teste la validit du coup avec les couleurs des cases voisines
bool KJeu :: bonneCouleur ( int colonne, int ligne, int col, int lig, int couleur )
{
   int colOppo = couleurOppose ( couleur ) ;

   // Dplacement du pion pour voir si un pion de l'autre couleur est  cot
   int c = colonne + col ;
   int l = ligne + lig ;

   // Si les pions ont la mm couleur, ce n'est pas bon
   if ( pions [ c ] [ l ].getCouleur ( ) != colOppo )
      return false ;

   // Tant que l'on ne trouve pas notre couleur
   while ( pions [ c ] [ l ].getCouleur (  ) == colOppo )
   {
      c += col ;
      l += lig ;
   }

   // Si c'est notre couleur donc c'est un coup valide
   if ( pions [ c ] [ l ].getCouleur ( ) == couleur )
      return true ;

   return false ;
}


// Suivant la couleur du pion, renvoi le score
int KJeu :: calculeScore ( int col )
{
   int result = 0 ;
   for ( int j = 1; j <= tailleDamier; j++ )
      for ( int i = 1; i <= tailleDamier; i++ )
      {
         if ( pions [ i ] [ j ].getCouleur ( ) == col )
            result ++ ;
      }
   return result ;
}


// Test sur notre couleur
bool KJeu :: coupLocal ( )
{
   if ( leCoup == maCouleur )
      return true ;
   return false ;
}


// Renvoie la couleur oppose d'un pion
int KJeu :: couleurOppose ( int couleur )
{
  if ( couleur == BLANC )
     return NOIR ;
  else
     return BLANC ;
}


// Mthode vrifiant si un coup est autoris ou non
bool KJeu :: deplacementValide ( int couleur, int colonne, int ligne )
{
   if ( leCoup == RIEN ) return false ;
   // Vrifie  ne pas la colonne ou la ligne
   if ( colonne < 1 || colonne > tailleDamier || ligne < 1 || ligne > tailleDamier ) return false;

   // Si un pion est deja dans la case
   if ( pions [ colonne ] [ ligne ].getCouleur ( ) != RIEN )  return false ;

   // Teste la couleur des pions des cases voisines
   if ( bonneCouleur ( colonne, ligne, -1, -1, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, 0, -1, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, 1, -1, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, -1, 0, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, 1, 0, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, -1, 1, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, 0, 1, couleur ) ) return true;
   if ( bonneCouleur ( colonne, ligne, 1, 1, couleur ) ) return true;

   return false ;
}


// Fait les coups dans toutes les directions possibles
void KJeu :: faitCoup ( int couleur, int colonne, int ligne )
{
   pions [ colonne ] [ ligne ].setCouleur ( couleur ) ;
   retournePion ( colonne, ligne, -1,  -1, couleur ) ;
   retournePion ( colonne, ligne, 0, -1, couleur ) ;
   retournePion ( colonne, ligne, 1,-1, couleur ) ;
   retournePion ( colonne, ligne, -1, 0, couleur ) ;
   retournePion ( colonne, ligne, 1, 0, couleur ) ;
   retournePion ( colonne, ligne, -1, 1, couleur ) ;
   retournePion ( colonne, ligne, 0, 1, couleur ) ;
   retournePion ( colonne, ligne, 1, 1, couleur ) ;
}


// Renvoie nom de la couleur
QString KJeu :: getNom ( int couleur )
{
   QString temp ;
   if ( couleur == 0 )
      temp = "NOIR" ;
   else if ( couleur == 1 )
      temp = "BLANC" ;
   else if ( couleur == 2 )
      temp = "CROIX" ;
   else if ( couleur == 3 )
      temp = "RIEN" ;
   return temp ;
}


// Renvoi le caractre suivant la position
int KJeu :: getPosColonne ( QString c )
{
   if ( QString :: compare ( c, "A" ) == 0 || QString :: compare ( c, "a" ) == 0 ) return 1 ;
   if ( QString :: compare ( c, "B" ) == 0 || QString :: compare ( c, "b" ) == 0 ) return 2 ;
   if ( QString :: compare ( c, "C" ) == 0 || QString :: compare ( c, "c" ) == 0 ) return 3 ;
   if ( QString :: compare ( c, "D" ) == 0 || QString :: compare ( c, "d" ) == 0 ) return 4 ;
   if ( QString :: compare ( c, "E" ) == 0 || QString :: compare ( c, "e" ) == 0 ) return 5 ;
   if ( QString :: compare ( c, "F" ) == 0 || QString :: compare ( c, "f" ) == 0 ) return 6 ;
   if ( QString :: compare ( c, "G" ) == 0 || QString :: compare ( c, "g" ) == 0 ) return 7 ;
   if ( QString :: compare ( c, "H" ) == 0 || QString :: compare ( c, "h" ) == 0 ) return 8 ;
   if ( QString :: compare ( c, "I" ) == 0 || QString :: compare ( c, "i" ) == 0 ) return 9 ;
   if ( QString :: compare ( c, "J" ) == 0 || QString :: compare ( c, "j" ) == 0 ) return 10 ;
   if ( QString :: compare ( c, "K" ) == 0 || QString :: compare ( c, "j" ) == 0 ) return 11 ;
   if ( QString :: compare ( c, "L" ) == 0 || QString :: compare ( c, "l" ) == 0 ) return 12 ;
   if ( QString :: compare ( c, "M" ) == 0 || QString :: compare ( c, "m" ) == 0 ) return 13 ;
   if ( QString :: compare ( c, "N" ) == 0 || QString :: compare ( c, "n" ) == 0 ) return 14 ;
   return -1 ;
}


// Renvoi le caractre suivant la position
QString KJeu :: getPosColonne ( int val )
{
   int temp = val - 1 ;
   QString pos ;
   for ( int i = 0; i < tailleDamier; i++ )
   {
      if ( temp == i )
      {
         pos = 'A' + i ;
         return pos ;
      }
   }
   return NULL ;
}


// Renvoi le caractre suivant la position
int KJeu :: getPosLigne ( QString c )
{
   if (c.length()>1)
      if (!c[1].isNumber())
         c=c.left(1);
      else if (!c[2].isNumber())
	c=c.left(2);

   if ( QString :: compare ( c, "1" ) == 0 ) return 1 ;
   if ( QString :: compare ( c, "2" ) == 0 ) return 2 ;
   if ( QString :: compare ( c, "3" ) == 0 ) return 3 ;
   if ( QString :: compare ( c, "4" ) == 0 ) return 4 ;
   if ( QString :: compare ( c, "5" ) == 0 ) return 5 ;
   if ( QString :: compare ( c, "6" ) == 0 ) return 6 ;
   if ( QString :: compare ( c, "7" ) == 0 ) return 7 ;
   if ( QString :: compare ( c, "8" ) == 0 ) return 8 ;
   if ( QString :: compare ( c, "9" ) == 0 ) return 9 ;
   if ( QString :: compare ( c, "10" ) == 0 ) return 10 ;
   if ( QString :: compare ( c, "11" ) == 0 ) return 11 ;
   if ( QString :: compare ( c, "12" ) == 0 ) return 12 ;
   if ( QString :: compare ( c, "13" ) == 0 ) return 13 ;
   if ( QString :: compare ( c, "14" ) == 0 ) return 14 ;
   return 0 ;
}


// Renvoi le caractre suivant la position
QString KJeu :: getPosLigne ( int val )
{
   if ( val == 1 ) return "1" ;
   if ( val == 2 ) return "2" ;
   if ( val == 3 ) return "3" ;
   if ( val == 4 ) return "4" ;
   if ( val == 5 ) return "5" ;
   if ( val == 6 ) return "6" ;
   if ( val == 7 ) return "7" ;
   if ( val == 8 ) return "8" ;
   if ( val == 9 ) return "9" ;
   if ( val == 10 ) return "10" ;
   if ( val == 11 ) return "11" ;
   if ( val == 12 ) return "12" ;
   if ( val == 13 ) return "13" ;
   if ( val == 14  ) return "14" ;
   return NULL ;
}


// Renvoi la position sur le damier
int KJeu :: getPosition ( int val )
{
   for ( int i = 1; i <= tailleDamier; i++ )
   {
      if ( val >= ( 25 * i ) && val <= ( 25 + ( 25 * i ) ) )
         return i ;
   }
   return -1 ;
}


// Initialise les 4 premiers pions sur le damier
void KJeu :: initDamier ( int t )
{
    leCoup = BLANC ;
    // maCouleur = couleur ;

   // Initialise le damier avec toutes les case vides
   for ( int j = 0; j < tailleDamier + 2; j++ )
      for ( int i = 0; i < tailleDamier + 2; i++ )
         pions [ i ] [ j ].setCouleur ( RIEN ) ;

   int i = ( t / 2 ) ;
   int j = ( t / 2 ) ;
   // Positionne les pions au centre du damier
   pions [ i ] [ j ].setCouleur ( BLANC ) ;
   pions [ i ] [ j + 1 ].setCouleur ( NOIR ) ;
   pions [ i + 1 ] [ j ].setCouleur ( NOIR ) ;
   pions [ i + 1 ] [ j + 1 ].setCouleur ( BLANC ) ;

   nouvoCoup ( ) ;
}


// Renvoie le nombre de dplacement possible
int KJeu :: nbPions ( int joueur )
{
   int count=0;
   for ( int j = 1; j <= tailleDamier; j++)
      for ( int i = 1; i <= tailleDamier; i++)
         if ( deplacementValide ( joueur, i, j ) )
	    count++;
   return count;
}


// Mthode faisant un nouveau coup
void KJeu :: nouvoCoup ( )
{
   leCoup = couleurOppose ( leCoup ) ;

   // Si le resultat est gal  0, le joueur doit passer son tour car aucune possibilit de dplacement
   if ( validesPositions ( ) == 0 )
   {
      if ( ( nbPions ( NOIR ) + nbPions ( BLANC ) ) == 0 )      // Plus personne ne peut faire de dplacement donc la partie est termine
      {
         repaint ( 25, 25, 25 * ( tailleDamier ), 25 * ( tailleDamier ), false ) ;
	 partieFinie=true;
         KMessageBox :: information ( this, "La partie est termine.", "Fin du jeu" ) ;
      }
      else
      {
         partieFinie = false;
         leCoup = couleurOppose ( leCoup ) ;
         KMessageBox :: information ( this, getNom ( couleurOppose ( leCoup ) ) + " est oblig de passer son tour !", "Attention" ) ;
         if ( couleurOppose ( leCoup )  == maCouleur )   // SI c'est moi, je passe mon tour
            emit sigNouvoCoup ( "t /os play " + id_partie + " PA");
         validesPositions (  ) ;
	 repaint ( 25, 25, 25 * ( tailleDamier ), 25 * ( tailleDamier  ), false ) ;
      }
   }
   else
      repaint ( 25, 25, 25 * ( tailleDamier ), 25 * ( tailleDamier ), false ) ;
}


// Mthode qui retourne un pion
void KJeu :: retournePion ( int colonne, int ligne, int col, int lig, int couleur )
{
   if ( bonneCouleur ( colonne, ligne, col, lig, couleur ) )
   {
      int c = colonne + col ;
      int l = ligne + lig ;
      int colOppo = couleurOppose ( couleur ) ;
      // Tant que c'est la couleur oppose, on tourne les pions
      while ( pions [ c ] [ l ].getCouleur ( ) == colOppo )
      {
         pions [ c ] [ l ].setCouleur ( couleur ) ;
         c += col ;
	 l += lig ;
      }
   }


}


// Initialise les variables de son login et de l'adversaire
void KJeu :: setJoueurs(QString l, QString a)
{
   login=l;
   adv=a;
}


// Valide un coup
void KJeu :: valideClick ( int c, int x, int y )
{
   if ( deplacementValide ( c, x, y ) )  // Le dplacement est autoris
   {
      faitCoup ( c, x, y ) ;

      coups = getPosColonne ( x ) + getPosLigne ( y );
      QString tmp ;
      tmp = "";

      if (numCoup < 10)
      	tmp="0" + tmp.setNum(numCoup);
      else
      	tmp=tmp.setNum(numCoup);

      itemNouvCoup = new QListViewItem ( listCoups, tmp, getNom ( c ), coups );
      listCoups -> insertItem ( itemNouvCoup );
      listCoups -> setMultiSelection (false);
      listCoups -> setSelected (itemNouvCoup, true);
      listCoups -> ensureItemVisible (itemNouvCoup);
      tmp = "" ;
      tmp = tmp.setNum ( calculeScore ( NOIR ) );
      if ( maCouleur == 0)
      	lblScoreNoir -> setText ( login + " (Noir) : " + tmp ) ;
      else
      	lblScoreNoir -> setText ( adv + " (Noir) : " + tmp ) ;

      tmp = "" ;
      tmp = tmp.setNum ( calculeScore ( BLANC ) ) ;
      if ( maCouleur == 0)
         lblScoreBlanc -> setText ( adv + " (Blanc) : " + tmp ) ;
      else
         lblScoreBlanc -> setText ( login + " (Blanc) : " + tmp ) ;
      numCoup ++ ;

      if ( couleurOppose ( c ) != maCouleur )
         emit sigNouvoCoup ( "t /os play " + id_partie + " " + coups ) ;

      nouvoCoup ( ) ;

      pthread_mutex_unlock(&mut);
   }
}


// Mthode calculant les positions valides et renvoyant le nombre de possiblit
int KJeu :: validesPositions ( )
{
   int count = 0 ;
   for ( int j = 1; j <= tailleDamier; j++ )
      for ( int i = 1; i <= tailleDamier; i++)
      {
         if ( deplacementValide ( leCoup, i, j ) )       // Dplacement autoris
         {
            pionsValides [ i ] [ j ] = true;
            count++ ;
         }
         else
	    pionsValides [ i ] [ j ] = false;
      }
   return count;
}


// Separe les champs
vector<QString> KJeu :: separe(QString source)
{
   int debut=0;
   vector<QString> res;

   for (uint i=0; i<source.length();i++)
   {
      if (source.mid(i,1)=="\n")
      {
         res.push_back(source.mid(debut, i-1-debut));
	 debut=i+1;
      }
   }

   return res;
}


// ------------------------------------------------------------------------------------------------ //
//			SLOT
// ------------------------------------------------------------------------------------------------ //

// Slot traitant les messages recus
void KJeu :: slotMsgRecu (QString s) //on gere ici les messages du serveur othello
{
   QString msg =s;
   QString chaine;
   vector<QString> v;
   int pos;
   int fin;
   int deb;
   int i;

   while(msg.find ("/os: ")!=-1) //utilise pour ant car il renvoi update dans la trame du debut de match
   {
      msg=msg.mid(msg.find("/os: ")+5, msg.length()-msg.find("/os: ")-5);
      if (msg.left(52) == "ERR your request doesn't fit the opponent's formula.")
      {
         emit sigFinPartie ("");
	 KMessageBox :: information ( this, "La personne ne peut pas accepter votre demande. \n Vous n'etes pas d'accord sur la nature du match.", "Erreur" ) ;
      }

      if (msg.left(40) == "ERR Player is not accepting new matches.")
      {
         emit sigFinPartie ("");
	 KMessageBox :: information ( this, "La personne n'accepte plus de nouveau match.", "Erreur" ) ;
      }

      if (msg.left(27)=="ERR you are not registered.")
      {
         // KMessageBox :: information ( this, "On n'est pas enregistr.", "Erreur" ) ;
	 //envoi->ajoute_tete(envoi->save());
      }

      if (msg.left(4)=="join")
      {
	 chaine = msg.mid(pos, msg.length()-pos);
	 v=separe(msg);

         for (i=0; v[i].mid(0,4)!="|PB["; i++);

	 if (demande)
	 {
	    if (v[i].mid(4, login.length())==login)
	    {
	       maCouleur = NOIR ;
	       KMessageBox :: information ( this, adv + " a accept le match, vous tes les pions NOIR.\nC'est  vous de commencer.", "Information" ) ;
	    }
	    else if (v[i].mid(4, adv.length())==adv)
	    {
	       maCouleur = BLANC ;
	       KMessageBox :: information ( this, adv + " a accept le match, vous tes les pions BLANC.\nAttendez votre tour pour jouer.", "Information" ) ;
	    }

	    emit sigAttente ("false");  // on cache la fenetre d'attente
	 }
	 else
	 {
	    if (v[i].mid(4, login.length())==login)
	    {
	       maCouleur = NOIR ;
	       KMessageBox :: information ( this, adv_temp + " a les pions BLANCS et vous avez les pions NOIRS.\nC'est  vous de commencer.", "Information" ) ;
	    }
	    else if (v[i].mid(4, adv_temp.length())==adv_temp)
	    {
	       maCouleur = BLANC ;
	       KMessageBox :: information ( this, adv_temp + " a les pions NOIRS et vous avez les pions BLANCS.\nAttendez votre tour pour jouer.", "Information" ) ;
	    }

	    for (i=0; v[i].mid(0,4)!="|BO["; i++);
	    for( fin=4; v[i].mid(fin, 1)!=" ";fin++);

	    tailleDamier=(v[i].mid(4, fin-4)).toInt();

            setGeometry ( ( KApplication :: desktop ( ) -> width ( ) - ( ( tailleDamier * 25 ) + 230 ) ) / 2,  ( KApplication :: desktop ( ) -> height ( ) - ( ( tailleDamier * 25 ) + 90 ) ) / 2, (tailleDamier * 25 ) + 230, ( tailleDamier * 25 ) + 90 ) ;
   	    setFixedSize ( ( tailleDamier * 25 ) + 230, ( tailleDamier * 25 ) + 85 ) ;
            setTaille ( tailleDamier );
         }

	 initDamier ( tailleDamier ) ;
	 i=0;
	 pos= v[i].find("join") + 5;

	 for(fin=pos; v[i].mid(fin, 1)!=" "; fin++);

	 id_partie=v[i].mid(pos, fin-pos);
	 show();
      }

      if (msg.left(6)=="update")
      {
         coup=true;

	 QString depl, sauvDepl, partie;
	 uint fin,deb;
	 int d=msg.find("update")+7;

	 for (deb=d; msg.mid(deb,1)!=" "; deb++);

	 partie =msg.mid(d, deb-d);
	 deb++;

         for (fin=deb;msg.mid(fin,1)!="/" && fin!=msg.length(); fin++);

	 depl =msg.mid(deb, fin-deb);

	 sauvDepl = depl ;

	 int retCol = getPosColonne ( depl.left ( 1 ) ) ;
	 int retLigne = getPosLigne ( depl.mid ( 1, sauvDepl.length ( ) -1 ) ) ;

	 valideClick ( leCoup, retCol, retLigne ) ;
      }

      if (msg.left(7)=="- match")
      {
	 if (msg.mid (8, id_partie.length())==id_partie)
	 {
	    pthread_mutex_lock(&mut);
	    partieFinie=true;
	    demande=false;
	 }
      }

      if (msg.left(3)=="-  " && (msg.find(login)!=-1) && msg.find(id_partie)!=-1 ) //&& msg.right(login.length())!=login)// on teste sur l'id de la partie car on accepte qu'une partie a la fois
      {
         if (demandeEnCours ==true && !annule)
	 {
	    KMessageBox :: information ( this, "L'adversaire a refus votre demande.", "Annulation de la demande" ) ;
	    emit sigAttente("false+fin");
	 }
	 demande=false; //on se met en attente d'une demande
	 id_partie=""; //notre partie est finie donc il n'y a plus d'id!!
      }

      if (msg.left(2)=="+ " && msg.mid(2,5)!="match" && (msg.find(login)!=-1) && id_partie=="" && !demande) //&& msg.right(login.length())!=login)// on teste sur l'id de la partie car on accepte qu'une partie a la fois
      {
   	 for(deb=2;msg.mid(deb,1)==" "; deb++);
	 for (fin=deb; msg.mid(fin,1)!= " "; fin++);
	 id_temp=msg.mid(deb, fin-deb);

	 for (deb=fin+1; msg.mid(deb,1)!= " "; deb++);
	 for (fin=deb+1; msg.mid(fin,1)!=" "; fin++);
	 adv_temp = msg.mid(deb+1, fin-deb-1);

	 for (deb=fin+1; msg.mid(deb,1)==" "; deb++);
	 for (deb=deb; msg.mid(deb,1)!=" "; deb++);
	 for (deb=deb+1; msg.mid(deb,1)==" "; deb++);
	 for (fin=deb; msg.mid(fin,1)!= " " ; fin++);
	 type_temp = msg.mid(deb, fin-deb);

	 emit sigDemande(id_temp,adv_temp,type_temp);
      }
      else if (msg.left(2)=="+ " && demande && msg.left(7)!="+ match")
      {
	 for(deb=2; msg.mid(deb,1)==" "; deb++);
	 for (fin=deb;msg.mid(fin,1)!=" "; fin++);
         id_temp=msg.mid(deb,fin-deb);
	 annule = false;
	 emit sigAttente (id_temp);
      }
   }		//fin du while
}


// Met  jour une partie
void KJeu :: majPartie (QString msg)
{
   QString depl, sauvDepl, partie;
   uint fin,deb;
   int d=msg.find("update")+7;

   for (deb=d; msg.mid(deb,1)!=" "; deb++);

   partie =msg.mid(d, deb-d);
   deb++;

   for (fin=deb;msg[fin]!='/' && msg[fin]!='\n'&& fin<=deb+3; fin++);

   depl =msg.mid(deb, fin-deb);
   sauvDepl = depl ;
   int retCol = getPosColonne ( depl.left ( 1 ) ) ;
   int retLigne = getPosLigne ( depl.mid ( 1, sauvDepl.length ( ) -1 ) ) ;
   valideClick ( leCoup, retCol, retLigne ) ;
}


// Click sur le bouton quitter
void KJeu :: slotbtnQuitter ( )
{
   if (partieFinie ==false)
   {
      QMessageBox mb ( "Confirmation", "Voulez vous vraiment quitter la partie ?", QMessageBox::Information, QMessageBox::Yes, QMessageBox::No, QMessageBox::Escape ) ;
      mb.setButtonText ( QMessageBox::Yes, "Oui" );
      mb.setButtonText ( QMessageBox::No, "Non" );
      mb.setFixedSize ( 300, 90 ) ;

      if ( mb.exec ( ) == QMessageBox::Yes )
      {
         emit sigFinPartie(id_partie);
         id_partie="";
         hide ( ) ;
         adv="";
         partieFinie=false;
         demandeEnCours=false;
      }
   }
   else
   {
      emit sigFinPartie ("");
      id_partie="";
      hide ( ) ;
      adv="";
      partieFinie=false;
      demandeEnCours=false;
   }
}


// Rpond  la demande
void KJeu :: slotRepDemande (int i , QString id)
{
   if (i==1)
   {
      adv=adv_temp;
      id_partie=id;
   }
}


// ------------------------------------------------------------------------------------------------ //
//			Mthode de la classe monPion
// ------------------------------------------------------------------------------------------------ //

// Suivant la couleur du pion, renvoi l'image
QImage monPion :: dessinePion ( int couleur )
{

   
   QImage image ;
   if ( couleur == NOIR )
      image.load (locate("appdata", "pics/noir.png") , 0 ) ;
   else if ( couleur == BLANC )
      image.load (locate("appdata", "pics/blanc.png") , 0 ) ;
   else if ( couleur == CROIX )
      image.load ( locate("appdata", "pics/croix.png"), 0 ) ;
   else if ( couleur == RIEN )
      image.load ( locate("appdata", "pics/fond.png"), 0 ) ;
   return image ;
}


// Demande
void KJeu :: set_demande(bool i)
{
   demande=i;
}

